package de.ozgcloud.admin.common;

import java.util.LinkedList;
import java.util.List;
import java.util.function.BooleanSupplier;
import java.util.function.Predicate;
import java.util.stream.Stream;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.Link;

import lombok.RequiredArgsConstructor;

public class CollectionModelBuilder<T> {

	private final Iterable<T> entities;

	private final List<Link> links = new LinkedList<>();

	private CollectionModelBuilder(Iterable<T> entities) {
		this.entities = entities;
	}

	public static <T> CollectionModelBuilder<T> fromEntities(Iterable<T> entities) {
		return new CollectionModelBuilder<>(entities);
	}

	public static <T> CollectionModelBuilder<T> fromEntities(Stream<T> entities) {
		return new CollectionModelBuilder<>(entities.toList());
	}

	public CollectionModelBuilder<T> addLink(Link link) {
		links.add(link);
		return this;
	}

	public ConditionalLinkAdder ifMatch(BooleanSupplier guard) {
		return new ConditionalLinkAdder(guard.getAsBoolean());
	}

	public ConditionalLinkAdder ifMatch(Predicate<? super Iterable<T>> predicate) {
		return new ConditionalLinkAdder(predicate.test(entities));
	}

	public CollectionModel<T> buildModel() {
		var builtModel = CollectionModel.of(entities);
		builtModel.add(links);
		return builtModel;
	}

	@RequiredArgsConstructor
	public class ConditionalLinkAdder {

		public final boolean conditionFulfilled;

		public CollectionModelBuilder<T> addLink(Link link) {
			if (conditionFulfilled) {
				links.add(link);
			}
			return CollectionModelBuilder.this;
		}
	}
}