package de.ozgcloud.admin.common;

import java.lang.reflect.InvocationTargetException;

import org.apache.commons.lang3.reflect.ConstructorUtils;

import com.fasterxml.jackson.databind.BeanProperty;
import com.fasterxml.jackson.databind.DeserializationContext;
import com.fasterxml.jackson.databind.JsonDeserializer;

import de.ozgcloud.common.errorhandling.TechnicalException;

public class LinkedResourceDeserializer extends AbstractLinkedResourceDeserializer {

	private static final long serialVersionUID = 1L;

	private LinkedResource annotation;

	protected LinkedResourceDeserializer() {
		super();
	}

	protected LinkedResourceDeserializer(BeanProperty beanProperty) {
		super(beanProperty);
		this.annotation = beanProperty.getAnnotation(LinkedResource.class);
	}

	@Override
	Object buildByBuilder(String url) {
		ObjectBuilder<?> builder;
		try {
			builder = ConstructorUtils.invokeConstructor(annotation.builder()).constructContextAware(getBeanProperty());
		} catch (NoSuchMethodException | IllegalAccessException | InvocationTargetException | InstantiationException e) {
			throw new TechnicalException("Error instanciating Builder.", e);
		}
		return builder.build(extractStringId(url));
	}

	@Override
	public JsonDeserializer<?> createContextual(DeserializationContext ctxt, BeanProperty property) {
		return new LinkedResourceDeserializer(property);
	}

}
