package de.ozgcloud.admin.keycloak;

import java.util.List;

import org.keycloak.admin.client.resource.GroupsResource;
import org.keycloak.representations.idm.GroupRepresentation;
import org.springframework.stereotype.Service;

import jakarta.ws.rs.core.Response;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
class KeycloakApiService {

	private final GroupsResource groupsResource;

	public List<GroupRepresentation> getAllGroups() {
		return groupsResource.groups("", 0, Integer.MAX_VALUE, false);
	}

	public String addGroup(GroupRepresentation groupRepresentation) {
		try (var response = groupsResource.add(groupRepresentation)) {
			return getCreatedResourceIdFromResponse(response);
		}
	}

	String getCreatedResourceIdFromResponse(Response response) {
		if (response.getStatus() == Response.Status.CREATED.getStatusCode()) {
			return extractResourceIdFromLocationHeader(response.getHeaderString("Location"));
		}
		throw new ResourceCreationException(
				"Failed to add group - got response with status %d (%s) and headers %s".formatted(response.getStatus(), response.getStatusInfo()
						.getReasonPhrase(), response.getHeaders()));
	}

	String extractResourceIdFromLocationHeader(String location) {
		return location.substring(location.lastIndexOf('/') + 1);
	}
}
