package de.ozgcloud.admin.organisationseinheit;

import org.springframework.hateoas.CollectionModel;
import org.springframework.hateoas.EntityModel;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PatchMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import lombok.RequiredArgsConstructor;

@RestController
@RequestMapping(OrganisationsEinheitController.PATH)
@RequiredArgsConstructor
public class OrganisationsEinheitController {

	static final String PATH = "/api/organisationseinheits"; // NOSONAR

	private final OrganisationsEinheitService organisationsEinheitService;

	private final OrganisationsEinheitModelAssembler assembler;

	@GetMapping
	public CollectionModel<EntityModel<OrganisationsEinheit>> getAll() {
		var organisationsEinheiten = organisationsEinheitService.getOrganisationsEinheiten();
		return assembler.toCollectionModel(organisationsEinheiten);
	}

	@GetMapping("/{id}")
	public EntityModel<OrganisationsEinheit> getById(@PathVariable String id) {
		return assembler.toModel(organisationsEinheitService.getOrganisationsEinheitById(id));
	}

	@GetMapping("/{id}/children")
	public CollectionModel<EntityModel<OrganisationsEinheit>> getChildren(@PathVariable String id) {
		var children = organisationsEinheitService.getChildren(id);
		return assembler.toChildrenCollectionModel(id, children);
	}

	@PatchMapping("/{id}")
	public EntityModel<OrganisationsEinheit> editOrganisationsEinheit(@PathVariable String id,
			@RequestBody OrganisationsEinheitPatch organisationsEinheitPatch) {
		var patched = organisationsEinheitService.editOrganisationsEinheit(id, organisationsEinheitPatch);
		return assembler.toModel(patched);
	}
}
