package de.ozgcloud.admin.organisationseinheit;

import java.util.List;
import java.util.Objects;

import org.springframework.data.rest.webmvc.ResourceNotFoundException;
import org.springframework.stereotype.Service;

import de.ozgcloud.common.errorhandling.TechnicalException;
import lombok.RequiredArgsConstructor;

@Service
@RequiredArgsConstructor
class OrganisationsEinheitService {

	private final OrganisationsEinheitRepository repository;

	public OrganisationsEinheit saveOrganisationsEinheit(OrganisationsEinheit organisationsEinheit) {
		return repository.save(organisationsEinheit);
	}

	public List<OrganisationsEinheit> getOrganisationsEinheiten() {
		var organisationsEinheiten = repository.findAllNotDeleted();
		return organisationsEinheiten.stream()
				.filter(organisationsEinheit -> Objects.isNull(organisationsEinheit.getParentId()))
				.map(rootOrganisationsEinheit -> addChildren(rootOrganisationsEinheit, organisationsEinheiten))
				.toList();
	}

	OrganisationsEinheit addChildren(OrganisationsEinheit rootOrganisationsEinheit, List<OrganisationsEinheit> allOrganisationsEinheiten) {
		return rootOrganisationsEinheit.toBuilder()
				.children(allOrganisationsEinheiten.stream()
						.filter(organisationsEinheit -> rootOrganisationsEinheit.getId().equals(organisationsEinheit.getParentId())).toList())
				.build();
	}

	public OrganisationsEinheit getOrganisationsEinheitById(String id) {
		return repository.findById(id).orElseThrow(() -> new ResourceNotFoundException("Organisationseinheit with id " + id + " not found"));
	}

	public List<OrganisationsEinheit> getChildren(String parentId) {
		return repository.findChildren(parentId);
	}

	public OrganisationsEinheit editOrganisationsEinheit(String id, OrganisationsEinheitPatch organisationsEinheitPatch) {
		return repository.findById(id)
				.map(organisationsEinheit -> patchOrganisationsEinheit(organisationsEinheit, organisationsEinheitPatch))
				.map(repository::save)
				.orElseThrow(() -> new TechnicalException("Organisationseinheit with id " + id + " not found"));
	}

	OrganisationsEinheit patchOrganisationsEinheit(OrganisationsEinheit organisationsEinheit, OrganisationsEinheitPatch patch) {
		return organisationsEinheit.toBuilder()
				.settings(
						organisationsEinheit.getSettings().toBuilder()
								.signatur(patch.getSignatur())
								.build())
				.build();
	}
}
