package de.ozgcloud.admin.common;

import java.time.Duration;

import org.springframework.boot.test.util.TestPropertyValues;
import org.springframework.context.ApplicationContextInitializer;
import org.springframework.context.ConfigurableApplicationContext;
import org.testcontainers.containers.wait.strategy.Wait;

import dasniko.testcontainers.keycloak.KeycloakContainer;
import lombok.extern.log4j.Log4j2;

@Log4j2
public class KeycloakInitializer implements ApplicationContextInitializer<ConfigurableApplicationContext> {

	private static final String AUTH_SERVER_URL = "ozgcloud.oauth2.auth-server-url";

	private static KeycloakContainer keycloakContainer;

	@Override
	public void initialize(ConfigurableApplicationContext applicationContext) {
		initContainer();
		setProperties(applicationContext);
	}

	@SuppressWarnings("resource")
	private void initContainer() {
		if (keycloakContainer == null) {
			LOG.info("Creating Keycloak-container...");
			keycloakContainer = new KeycloakContainer().withRealmImportFile("keycloak/realm-export.json").withVerboseOutput();
		}
		if (!keycloakContainer.isRunning()) {
			LOG.info("Starting Keycloak-container...");
			keycloakContainer.setWaitStrategy(
					Wait.forLogMessage(".*message\":\"started.*", 1).withStartupTimeout(Duration.ofMinutes(3)));
			keycloakContainer.start();
			LOG.info("Keycloak-container started");
		}
	}

	private void setProperties(ConfigurableApplicationContext applicationContext) {
		LOG.info("Keycloak URL: {}", keycloakContainer.getAuthServerUrl());

		TestPropertyValues.of(AUTH_SERVER_URL + "=" + keycloakContainer.getAuthServerUrl()).applyTo(applicationContext);

	}
}
