/*
 * Copyright (C) 2022 Das Land Schleswig-Holstein vertreten durch den
 * Ministerpräsidenten des Landes Schleswig-Holstein
 * Staatskanzlei
 * Abteilung Digitalisierung und zentrales IT-Management der Landesregierung
 *
 * Lizenziert unter der EUPL, Version 1.2 oder - sobald
 * diese von der Europäischen Kommission genehmigt wurden -
 * Folgeversionen der EUPL ("Lizenz");
 * Sie dürfen dieses Werk ausschließlich gemäß
 * dieser Lizenz nutzen.
 * Eine Kopie der Lizenz finden Sie hier:
 *
 * https://joinup.ec.europa.eu/collection/eupl/eupl-text-eupl-12
 *
 * Sofern nicht durch anwendbare Rechtsvorschriften
 * gefordert oder in schriftlicher Form vereinbart, wird
 * die unter der Lizenz verbreitete Software "so wie sie
 * ist", OHNE JEGLICHE GEWÄHRLEISTUNG ODER BEDINGUNGEN -
 * ausdrücklich oder stillschweigend - verbreitet.
 * Die sprachspezifischen Genehmigungen und Beschränkungen
 * unter der Lizenz sind dem Lizenztext zu entnehmen.
 */
package de.ozgcloud.admin.common;

import static org.assertj.core.api.Assertions.*;
import static org.mockito.Mockito.*;

import java.util.UUID;

import org.junit.jupiter.api.BeforeEach;
import org.junit.jupiter.api.DisplayName;
import org.junit.jupiter.api.Nested;
import org.junit.jupiter.api.Test;
import org.junit.jupiter.params.ParameterizedTest;
import org.junit.jupiter.params.provider.NullAndEmptySource;
import org.mockito.Mock;
import org.springframework.context.ApplicationContext;
import org.springframework.core.env.Environment;
import org.springframework.web.bind.annotation.RequestMapping;

import lombok.Builder;

class ModelBuilderTest {

	@DisplayName("Add link by annotation if missing")
	@Nested
	class TestAddLinkByAnnotationIfMissing {

		private static final String USER_MANAGER_URL = "http://localhost";
		private static final String USER_MANAGER_PROFILE_TEMPLATE = "/api/profile/%s";

		@Mock
		private ApplicationContext context;

		private TestEntity entity = TestEntityTestFactory.create();

		@Test
		void shouldHaveAddLinkByLinkedResource() {
			var model = ModelBuilder.fromEntity(entity).buildModel();

			assertThat(model.getLink(TestController.FILE_REL).get().getHref()).isEqualTo(TestController.PATH + "/" + TestEntityTestFactory.FILE);
		}

		@ParameterizedTest
		@NullAndEmptySource
		void shouldNotAddLinkByLinkedRessourceIfFieldValueIsNotSet(String fileId) {
			var model = ModelBuilder.fromEntity(TestEntityTestFactory.createBuilder().file(fileId).build()).buildModel();

			assertThat(model.getLink(TestController.FILE_REL)).isEmpty();
		}
	}
}

@Builder
class TestEntity {

	@LinkedResource(controllerClass = TestController.class)
	private String file;
}

@RequestMapping(TestController.PATH)
class TestController {

	static final String PATH = "/api/test";

	static final String USER_REL = "user";
	static final String FILE_REL = "file";

}

class TestEntityTestFactory {

	static final String USER = UUID.randomUUID().toString();
	static final String FILE = UUID.randomUUID().toString();

	public static TestEntity create() {
		return createBuilder().build();
	}

	public static TestEntity.TestEntityBuilder createBuilder() {
		return TestEntity.builder()
				.file(FILE);
	}
}
