package de.ozgcloud.admin.keycloak;

import java.util.List;
import java.util.Map;
import java.util.UUID;

import org.keycloak.representations.idm.GroupRepresentation;

import com.thedeanda.lorem.LoremIpsum;

class GroupRepresentationTestFactory {

	public static final String ORGANIZATIONS_EINHEIT_ID_ATTRIBUTE = "organisationseinheitId";

	public static final String ID = UUID.randomUUID().toString();
	public static final String PARENT_ID = UUID.randomUUID().toString();
	public static final String NAME = LoremIpsum.getInstance().getName();
	public static final String ORGANISATIONS_EINHEIT_ID = UUID.randomUUID().toString();
	public static final Map<String, List<String>> ATTRIBUTES = Map.of(ORGANIZATIONS_EINHEIT_ID_ATTRIBUTE, List.of(ORGANISATIONS_EINHEIT_ID));

	public static final String SUB_GROUP_ID = UUID.randomUUID().toString();
	public static final String SUB_GROUP_NAME = LoremIpsum.getInstance().getName();
	public static final String SUB_GROUP_ORGANISATIONS_EINHEIT_ID = UUID.randomUUID().toString();
	public static final Map<String, List<String>> SUB_GROUP_ATTRIBUTES = Map.of(ORGANIZATIONS_EINHEIT_ID_ATTRIBUTE, List.of(SUB_GROUP_ORGANISATIONS_EINHEIT_ID));

	public static GroupRepresentation create() {
		return create(true);
	}

	public static GroupRepresentation create(String name) {
		var groupRepresentation = create();
		groupRepresentation.setName(name);
		return groupRepresentation;
	}

	public static GroupRepresentation createWithoutId(String name) {
		var groupRepresentation =  create(false);
		groupRepresentation.setName(name);
		return groupRepresentation;
	}

	public static GroupRepresentation create(boolean withId) {
		var groupRepresentation = new GroupRepresentation();
		if (withId) {
			groupRepresentation.setId(ID);
		}
		groupRepresentation.setParentId(PARENT_ID);
		groupRepresentation.setName(NAME);
		groupRepresentation.setAttributes(ATTRIBUTES);
		groupRepresentation.setSubGroups(List.of(createSubGroup(withId)));
		return groupRepresentation;
	}

	private static GroupRepresentation createSubGroup(boolean withId) {
		var groupRepresentation = new GroupRepresentation();
		if (withId) {
			groupRepresentation.setId(SUB_GROUP_ID);
		}
		groupRepresentation.setParentId(ID);
		groupRepresentation.setName(SUB_GROUP_NAME);
		groupRepresentation.setAttributes(SUB_GROUP_ATTRIBUTES);
		return groupRepresentation;
	}
}
